import { Http, Metadata } from '@zeedhi/core';
import { Indicators } from '@/helper/Indicators';
import SharedInfo from '@/helper/SharedInfo';
import { DashboardGrid } from '@zeedhi/zd-dashboard-grid';
import { ZdGrid, ZdTabs, ZdButtonGroup } from '@zeedhi/vuetify';
import { Utilities } from '@/util/Utilities';
import { GenericValueController } from '@/controllers/GenericValueController';
import ChangeLabel from '@/helper/Labels';

export class DashboardController {
  private _accessIndicators: number[];

  public editingModeDash = false;
  
  public secondLoad = 0;
  
  public homePanel = null;
  
  public backupCardsPositionGU = [];

  public backupCardsPositionGO = [];

  public backupCardsPositionTD = [];

  public backupCardsPositionTA = [];
 
  public get accessIndicators(): number[] {
    /* eslint no-underscore-dangle: 0 */
    return this._accessIndicators;
  }

  public set accessIndicators(accessIndicators: number[]) {
    /* eslint no-underscore-dangle: 0 */
    this._accessIndicators = accessIndicators;
  }
  
  /**
 * Retrieve and populate a data grid component with data, performing necessary formatting and updates.
 *
 * This function is responsible for retrieving data and populating a data grid component with it, while
 * performing necessary formatting and updates. The data is fetched and then is processed to format numerical
 * values and apply necessary label changes. Additionally, the function updates the component's data source and
 * applies filters if applicable.
 *
 * @param {Object} component - The specific data grid component to be populated and updated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public getDataGrid({ component }) {
    const formatting = new Utilities();
    const genericValues = new GenericValueController();
    const genericTest = localStorage.getItem('GENERIC_VALUE');
    const routeName = component.children[0].datasourceName;
    const sendObject = {
      filter: [],
    };
    let dayOfWeekProc;
    let data;
    
    if (genericTest === 'false' || genericTest === null) {
      Http.post(`/${routeName}`, sendObject).then((response) => {
        /* eslint no-console: ["error", { allow: ["warn", "error", "info", "log"] }] */
        data = response.data.dataset[routeName].map((obj) => {
          Object.keys(obj).forEach((key) => {
            if (typeof obj[key] === 'number') {
              if (key === 'NRDTPROC') {
                dayOfWeekProc = obj[key];
              } else if (key !== 'NRDIA') {
                if (key === 'PERCENTORCADO' || key === 'PERCENTREAL') {
                  obj[key] = `${formatting.formatValue('int', obj[key])}%`;
                } else {
                  obj[key] = formatting.formatValue('float', obj[key]);
                }
              }
            } else if (obj[key] instanceof Date) {
              console.log(obj[key]);
            } else if (key === 'NMITEM') {
              obj[key] = ChangeLabel.getLabel(obj[key]);
            }
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            }
          });
        
          return obj;
        });
        
        component.datasource.updateData(data);
        if (routeName === 'todayDeliveries') {
          const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
          
          zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
          component.datasource.setFilter({
            NRDIA: dayOfWeekProc,
          });
        }
      });
    } else {
      data = genericValues.valueGrid[component.name].map((obj) => {
        Object.keys(obj).forEach((key) => {
          if (typeof obj[key] === 'number') {
            if (key === 'NRDTPROC') {
              dayOfWeekProc = obj[key];
            } else if (key !== 'NRDIA') {
              if (key === 'PERCENTORCADO' || key === 'PERCENTREAL') {
                obj[key] = `${formatting.formatValue('int', obj[key])}%`;
              } else {
                obj[key] = formatting.formatValue('float', obj[key]);
              }
            }
          } else if (obj[key] instanceof Date) {
            console.log(obj[key]);
          } else if (key === 'NMITEM') {
            obj[key] = ChangeLabel.getLabel(obj[key]);
          }
        });
      
        return obj;
      });
      
      component.datasource.updateData(data);
      if (routeName === 'todayDeliveries') {
        const zdgroupbutton = Metadata.getInstance<ZdButtonGroup>('daysWeek');
        
        zdgroupbutton.selectedButtons = dayOfWeekProc - 1;
        component.datasource.setFilter({
          NRDIA: dayOfWeekProc,
        });
      }
    }
  }
  
  /**
 * Populate a card list component with visible cards based on user access indicators.
 *
 * This function is responsible for populating a card list component with visible cards based on the user's access
 * indicators. It awaits necessary asynchronous operations before executing. The function retrieves a list of
 * visible cards for the specific component from predefined indicator lists. The cards are filtered based on the
 * user's access indicators and then sorted. Finally, the component's cards are updated with the visible and sorted cards.
 *
 * @param {Object} component - The specific card list component to be populated.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public async cardList({ component }) {
    component.cards = [];
    // await this.accessibleIndicators();
    await SharedInfo.waitMyBrother();
    const indic = new Indicators();
    const visible = this.accessIndicators;
    let visibleCards = [];
    try {
      switch (component.name) {
        case 'dashGU': visibleCards = this.sortCards(indic.cardListGU.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          this.backupCardsPositionGU = JSON.parse(JSON.stringify(visibleCards));
          break;
        case 'dashGO': visibleCards = this.sortCards(indic.cardListGO.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          this.backupCardsPositionGO = JSON.parse(JSON.stringify(visibleCards));
          break;
        case 'dashTD': visibleCards = this.sortCards(indic.cardListTD.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          this.backupCardsPositionTD = JSON.parse(JSON.stringify(visibleCards));
          break;
        case 'dashTA': visibleCards = this.sortCards(indic.cardListTA.filter((obj) => (
          visible.includes(obj.cardId)
        )), component.name);
          this.backupCardsPositionTA = JSON.parse(JSON.stringify(visibleCards));
          break;
        default: visibleCards = [];
      }
      
      component.cards = visibleCards;
    } catch (error) {
      // Adicionar mensagem de erro
      component.cards = [];
    }
  }
  
  /**
 * Retrieve accessible indicators for the user and set additional data.
 *
 * This function is responsible for retrieving accessible indicators for the user and setting additional data.
 * It performs an request to fetch visible indicators and sets the accessed indicators and home panel.
 * The function ensures that the accessible indicators are retrieved only once and then cached for future use.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public async accessibleIndicators() {
    const indic = new Indicators();
    const sendObject = {
      filter: [],
    };
    
    try {
      if (typeof this.accessIndicators === 'undefined') {
        const routeName = 'visibleIndicators';
        const routePanel = 'homePanel';
        const response = await Http.post('/visibleIndicators', sendObject);
        this.accessIndicators = response.data.dataset[routeName].concat(indic.auxDash);
        this.homePanel = response.data.dataset[routePanel][0].HOMEPANEL;
      }
    } catch (error) {
      // Adicionar mensagem de erro
    }
  }
  
  /**
 * Determine and set the visibility of panels and tabs based on accessible indicators and other data.
 *
 * This function is responsible for determining and setting the visibility of panels and tabs based on the
 * user's accessible indicators and other data. It awaits necessary asynchronous operations before executing.
 * The function iterates through the component's tabs and checks each tab's associated accessible indicators
 * to determine if the panel should be visible. It also sets the active tab value based on local storage
 * and determines the overall visibility of the dashboard. Finally, it updates the component's state.
 *
 * @param {Object} component - The specific component representing the dashboard to configure visibility for.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public async visiblePanel({ component }) {
    await this.accessibleIndicators();
    const indic = new Indicators();
    
    let panelVisible = false;
    let dashVisible = false;
    
    component.tabs.forEach((obj) => {
      switch (obj.name) {
        case 'unitManager': panelVisible = indic.cardsGU.some((item) => this.accessIndicators.includes(item));
          break;
        case 'operationalManager': panelVisible = indic.cardsGO.some((item) => this.accessIndicators.includes(item));
          break;
        case 'costAnalysis': panelVisible = indic.cardsTD.some((item) => this.accessIndicators.includes(item));
          break;
        case 'tracking': panelVisible = indic.cardsTA.some((item) => this.accessIndicators.includes(item));
          break;
        default: panelVisible = false;
      }
      obj.isVisible = panelVisible;
      dashVisible = dashVisible || panelVisible;
    });
    const activePanel = localStorage.getItem('ACTIVE_PANEL');
    component.activeTabValue = activePanel !== 'null' ? activePanel : this.homePanel;
    localStorage.setItem('ACTIVE_PANEL', null);
    component.isVisible = dashVisible;
  }
  
  public cancelEditOrder() {
    this.returnToStartPosition();
  }
  
  /**
 * @param {Object} components
 *
 * @access public
 * @author Grazielle Antunes
 */
  public returnToStartPosition() {
    const namePanel = this.returnActiveTab();
    const dash = Metadata.getInstance<DashboardGrid>(namePanel);
    
    switch (namePanel) {
      case 'dashGU': dash.cards = JSON.parse(JSON.stringify(this.backupCardsPositionGU));
        break;
      case 'dashGO': dash.cards = JSON.parse(JSON.stringify(this.backupCardsPositionGO));
        break;
      case 'dashTD': dash.cards = JSON.parse(JSON.stringify(this.backupCardsPositionTD));
        break;
      case 'dashTA': dash.cards = JSON.parse(JSON.stringify(this.backupCardsPositionTA));
        break;
      default: dash.cards = [];
    }

    this.secondLoad -= 1;
    this.editingModeDash = false;
    this.enabledTabsDashboard(false);
  }

  public returnActiveTab() {
    let panel = [];
    const tabs = Metadata.getInstance<ZdTabs>('dashboardNext');
    const activeTab = tabs.activeTab;
    switch (activeTab) {
      case 0: panel = ['dashGU', 0];
        break;
      case 1: panel = ['dashGO', 1];
        break;
      case 2: panel = ['dashTD', 2];
        break;
      case 3: panel = ['dashTA', 3];
        break;
      default: panel = ['', ''];
    }
    return panel[0];
  }

  /**
 * Confirm and save the edited order of dashboard components.
 *
 * This function is responsible for confirming and saving the edited order of dashboard components.
 * It determines the panel to which the edited order belongs based on the component's name, and then
 * constructs a new order string using the card layout information. The new order is sent to the server
 * for storage along with the selected panel. After successful submission, it updates the active panel
 * in local storage and disables the dashboard tabs.
 *
 * @param {Object} components - The components containing the edited dashboard order information.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public confirmEditOrder(components) {
    let panel = [];
    let dash;
    switch (components.component.name) {
      case 'confirmEditOrderGU': panel = ['dashGU', 0];
        dash = Metadata.getInstance<DashboardGrid>(panel[0]);
        this.backupCardsPositionGU = JSON.parse(JSON.stringify(dash.cards));
        break;
      case 'confirmEditOrderGO': panel = ['dashGO', 1];
        dash = Metadata.getInstance<DashboardGrid>(panel[0]);
        this.backupCardsPositionGO = JSON.parse(JSON.stringify(dash.cards));
        break;
      case 'confirmEditOrderAC': panel = ['dashTD', 2];
        dash = Metadata.getInstance<DashboardGrid>(panel[0]);
        this.backupCardsPositionTD = JSON.parse(JSON.stringify(dash.cards));
        break;
      case 'confirmEditOrderTA': panel = ['dashTA', 3];
        dash = Metadata.getInstance<DashboardGrid>(panel[0]);
        this.backupCardsPositionTA = JSON.parse(JSON.stringify(dash.cards));
        break;
      default: panel = ['', ''];
    }
    if (panel[0] !== '') {
      // const dash = Metadata.getInstance<DashboardGrid>(panel[0]);
      /* const newOrder = dash.cards.map((obj) => `${obj.i}|${obj.h}|${obj.w}|${obj.x}|${obj.y}`).join(','); */
      const newOrder = dash.cards.map((obj) => `${obj.i}|${obj.x}|${obj.y}`).join(',');
      const sendObject = {
        orderIndicDash: newOrder,
        selectedPanel: panel[0],
      };
  
      Http.post('/changeOrderDash', sendObject).then(() => {
        localStorage.setItem('ACTIVE_PANEL', panel[1]);
        this.enabledTabsDashboard(false);
      });
    }
    this.editingModeDash = false;
  }
  
  /**
 * Sort and arrange dashboard cards based on saved order for a specific panel.
 *
 * This function is responsible for sorting and arranging dashboard cards based on a saved order
 * for a specific panel. It takes an array of cards and the name of the panel as parameters. The
 * function retrieves the saved card order for the panel, then maps and updates the position of
 * each card according to the saved order. If no saved order is found, the function returns the
 * cards as-is. The sorted cards array is returned as the result.
 *
 * @param {Array} cards - An array of dashboard cards to be sorted.
 * @param {string} panel - The name of the panel for which to sort the cards.
 *
 * @returns {Array} An array of sorted dashboard cards.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  sortCards(cards: any, panel: string): any {
    const cardOrder = SharedInfo.getOrderPanel(panel);
    let cardsTmp;
    if (cardOrder) {
      const savedCards = cardOrder.split(',').map((item) => {
        const [i, x, y] = item.split('|').map(parseFloat);
        return {
          i, x, y,
        };
      });
      
      cardsTmp = cards.map((obj) => {
        const auxCardObj = savedCards.find((savedCardObj) => savedCardObj.i === obj.i);
      
        if (auxCardObj) {
          /* obj.h = auxCardObj.h; */
          /* obj.w = auxCardObj.w; */
          obj.x = auxCardObj.x;
          obj.y = auxCardObj.y;
        }

        return obj;
      });
    } else {
      cardsTmp = cards;
    }
    return cardsTmp;
  }

  /**
 * Handle the display and editing mode of dashboard cards.
 *
 * This function is responsible for handling the display and editing mode of dashboard cards.
 * It takes the component as a parameter and checks the current state of the dashboard cards.
 * If there are cards present and it's the second load, the function enables the editing mode
 * for the dashboard and enables the dashboard tabs for interaction. If it's not the second load,
 * the function increments the secondLoad counter. If there are no cards, no action is taken.
 *
 * @param {Object} component - The specific component representing the dashboard.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public handlingCards({ component }) {
    if (component.cards.length !== 0) {
      if (this.homePanel === 0) {
        if (this.secondLoad === 2) {
          this.editingModeDash = true;
          this.enabledTabsDashboard(true);
        } else {
          this.secondLoad += 1;
        }
      } else if (this.homePanel !== 0) {
        if (this.secondLoad === 3) {
          this.editingModeDash = true;
          this.enabledTabsDashboard(true);
        } else {
          this.secondLoad += 1;
        }
      }
    }
  }
  
  /**
 * Enable or disable dashboard tabs based on editing mode.
 *
 * This function is responsible for enabling or disabling dashboard tabs based on the editing mode.
 * It takes a parameter 'editModeTab' which determines whether the tabs should be enabled or disabled.
 * The function retrieves the instance of the dashboard tabs, identifies the currently active tab, and
 * iterates through all tabs except the active one to set their 'disabled' property according to the
 * provided 'editModeTab' value.
 *
 * @param {boolean} editModeTab - A boolean indicating whether to enable or disable the tabs.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public enabledTabsDashboard(editModeTab) {
    const tabs = Metadata.getInstance<ZdTabs>('dashboardNext');
    const activeTab = tabs.activeTab;
    tabs.tabs.forEach((i) => {
      if (tabs.tabs[activeTab].name !== i.name) {
        i.disabled = editModeTab;
      }
    });
  }
  
  get editingMode() {
    return this.editingModeDash;
  }
  
  /**
 * Magic that make the Dashboard work, don't delete.
 *
 * @author Anonymous
 */
  public showDescription() {
    
  }
  
  /**
 * Configure the dashboard panels and indicator limits.
 *
 * This function is responsible for configuring the dashboard panels and indicator limits.
 * It checks if a single run is allowed (as determined by SharedInfo.getSingleRun) and performs
 * panel configuration and indicator limit setup. The function sends a POST request to '/panelConfig'
 * to retrieve configuration data. If the single run is allowed, the loaded panel order, indicator
 * limits, and panel configurations are set using SharedInfo methods. If loadingSolve is defined
 * in SharedInfo, it is called. Finally, the editing mode for the dashboard is turned off and
 * dashboard tabs are disabled.
 *
 * @access public
 * @author Jefferson Gouvea
 */
  public async panelConfig() {
    if (SharedInfo.getSingleRun) {
      await this.accessibleIndicators();
      const sendObject = {
        filter: [],
      };

      try {
        const response = await Http.post('/panelConfig', sendObject);
        const panelConfig = response.data.dataset.panelConfig[0];
        const indicConfig = response.data.dataset.indicConfig;
      
        indicConfig.forEach((obj) => {
          SharedInfo.setLimit(obj.NRINDICADORTF, obj.VRLIMMETA, obj.VRLIMMINIMO);
        });
        SharedInfo.setOrderPanel(
          panelConfig.loaded,
          panelConfig.DASHGU,
          panelConfig.DASHGO,
          panelConfig.DASHTD,
          panelConfig.DASHTA,
        );
      
        if (SharedInfo.loadingSolve) {
          SharedInfo.loadingSolve();
        }
      } catch (error) {
        console.log('teste');
      }
    }
    this.editingModeDash = false;
    this.enabledTabsDashboard(false);
  }
  
  /**
 * Filter data in the "Today Deliveries" grid by selected day of the week.
 *
 * This function is responsible for filtering data in the "Today Deliveries" grid based on the
 * selected day of the week. It takes a parameter 'components' which contains the specific
 * component that triggered the function. Depending on the component's name, it identifies the
 * corresponding day of the week and filters the data in the "Today Deliveries" grid accordingly.
 *
 * @param {Object} components - The specific component that triggered the function.
 *
 * @access public
 * @Jefferson Gouvea
 */
  public daysWeek(components) {
    const todayDeliveriesGrid = Metadata.getInstance<ZdGrid>('todayDeliveriesGrid');
    let nrDay;
    
    switch (components.component.name) {
      case 'DOM': nrDay = 1;
        break;
      case 'SEG': nrDay = 2;
        break;
      case 'TER': nrDay = 3;
        break;
      case 'QUA': nrDay = 4;
        break;
      case 'QUI': nrDay = 5;
        break;
      case 'SEX': nrDay = 6;
        break;
      default: nrDay = 7;
    }
    
    todayDeliveriesGrid.datasource.setFilter({
      NRDIA: nrDay,
    });
  }

  public loadingDash() {
    this.secondLoad = 0;
  }
  
  public executeProcedure() {
    Http.post('/executeProcedureTECFOODDSB');
  }
}
